#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

# shellcheck disable=2154,2153,2046,2034

readonly SYS_MEM="$(awk '/MemTotal/ {
    print int($2 / 1024)"M"
}' /proc/meminfo)"

readonly LOCALES="$(awk '/\.UTF-8/ {
    gsub(/# .*|#/, "")
    if ($1) {
        print $1 " -"
    }
}' /etc/locale.gen)"

readonly CMAPS="$(find /usr/share/kbd/keymaps -name '*.map.gz' | awk '{
    gsub(/\.map\.gz|.*\//, "")
    print $1 " -"
}')"

mnt_menu()
{
    # prepare partition list PARTS for dialog
    lvm_detect
    umount_dir $MNT
    find_partitions 'part|lvm|crypt' || return 1

    # remove boot partition from dialog list if we auto partitioned one
    [[ $BOOT_PART != "" ]] && decr_count "$BOOT_PART"

    select_root_partition || return 1

    if [[ $BOOT_PART ]]; then
        infobox "$_PrepMount" "\nUsing boot partition: $BOOT_PART\n" 1
    elif [[ $SYS == "UEFI" ]]; then
        select_efi_partition || { BOOT_PART=""; return 1; }
    elif (( COUNT > 0 )); then
        select_boot_partition || { BOOT_PART=""; return 1; }
    fi

    setup_boot || return 1
    select_swap || return 1
    select_extra_partitions || return 1
    return 0
}

select_swap()
{
    # Ask user to select partition or create swapfile
    tput civis
    if ! SWAP_PART="$(menubox "$_SelSwpSetup" "$_SelSwpBody" 0 0 0 "$_SelSwpNone" "-" "$_SelSwpFile" "$SYS_MEM" $PARTS)" || [[ $SWAP_PART == "$_SelSwpNone" ]]; then
        SWAP_PART=""
        return 0
    fi

    if [[ $SWAP_PART == "$_SelSwpFile" ]]; then
        tput cnorm
        local i=0
        while ! [[ ${SWAP_SIZE:0:1} =~ [1-9] && ${SWAP_SIZE: -1} =~ (M|G) ]]; do
            (( i > 0 )) && msgbox "$_SelSwpSetup Error" "\n$_SelSwpErr $SWAP_SIZE\n"
            if ! SWAP_SIZE="$(getinput "$_SelSwpSetup" "$_SelSwpSize" "$SYS_MEM")"; then
                SWAP_PART=""
                SWAP_SIZE=""
                break
                return 0
            fi
            ((i++))
        done

        enable_swap "$MNT/swapfile"
        SWAP_PART="/swapfile"
    else
        enable_swap "$SWAP_PART"
        decr_count "$SWAP_PART"
        SWAP_SIZE="$(lsblk -lno SIZE $SWAP_PART)"
    fi
    return 0
}

select_mountpoint()
{
    tput cnorm
    if ! EXTRA_MNT="$(getinput "$_PrepMount $part" "$_ExtPartBody1 /home /var\n" "/")"; then
        return 1
    fi

    # bad mountpoint
    if [[ ${EXTRA_MNT:0:1} != "/" || ${#EXTRA_MNT} -le 1 || $EXTRA_MNT =~ \ |\' ]]; then
        msgbox "$_ErrTitle" "$_ExtErrBody"
        select_mountpoint || return 1
    fi
    return 0
}

select_mount_opts()
{
    local part="$1"
    local fs="$2"
    local title="${fs^} Mount Options"
    local opts
    opts="${FS_OPTS[$fs]}"
    ssd "$part" && opts=$(sed 's/discard - off/discard - on/' <<< "$opts")

    tput civis
    if ! MNT_OPTS="$(dialog --cr-wrap --stdout --backtitle "$BT" --title " $title " \
        --checklist "$_MntBody" 0 0 0 $opts | sed 's/ /,/g; $s/,$//')" || [[ $MNT_OPTS == "" ]]; then
        return 1
    fi

    if ! yesno "$title" "$_MntConfBody $MNT_OPTS\n"; then
        select_mount_opts "$part" "$fs" || return 1
    fi

    return 0
}

select_filesystem()
{
    local part="$1"
    local fs

    local cur_fs
    cur_fs="$(lsblk -lno FSTYPE $part)"

    local title="\nSelect which filesystem you want to use for $part\n\nPartition Name:      "

    tput civis

    if [[ $cur_fs && $part != "$ROOT_PART" ]]; then
        fs="$(menubox "$_FSTitle: $part" \
            "${title}${part}\nExisting Filesystem: ${cur_fs}$_FSBody" 0 0 0 \
            "$_Skip"   "-" \
            "ext4"     "${FS_CMDS[ext4]}" \
            "ext3"     "${FS_CMDS[ext3]}" \
            "ext2"     "${FS_CMDS[ext2]}" \
            "vfat"     "${FS_CMDS[vfat]}" \
            "ntfs"     "${FS_CMDS[ntfs]}" \
            "f2fs"     "${FS_CMDS[f2fs]}" \
            "jfs"      "${FS_CMDS[jfs]}" \
            "nilfs2"   "${FS_CMDS[nilfs2]}" \
            "reiserfs" "${FS_CMDS[reiserfs]}" \
            "xfs"      "${FS_CMDS[xfs]}")"
    else
        fs="$(menubox "$_FSTitle: $part" \
            "${title}${part}$_FSBody" 0 0 0 \
            "ext4"     "${FS_CMDS[ext4]}" \
            "ext3"     "${FS_CMDS[ext3]}" \
            "ext2"     "${FS_CMDS[ext2]}" \
            "vfat"     "${FS_CMDS[vfat]}" \
            "ntfs"     "${FS_CMDS[ntfs]}" \
            "f2fs"     "${FS_CMDS[f2fs]}" \
            "jfs"      "${FS_CMDS[jfs]}" \
            "nilfs2"   "${FS_CMDS[nilfs2]}" \
            "reiserfs" "${FS_CMDS[reiserfs]}" \
            "xfs"      "${FS_CMDS[xfs]}")"
    fi

    [[ $fs == "$_Skip" ]] && return 0 || { [[ $fs == "" ]] && return 1; }

    if yesno "$_FSTitle" "\nFormat $part as $fs?\n" "Format" "Go Back"; then
        format $part $fs
    else
        select_filesystem $part || return 1
    fi

    return 0
}

select_efi_partition()
{
    tput civis
    if (( COUNT == 1 )); then
        BOOT_PART="$(awk 'NF > 0 {print $1}' <<< "$PARTS")"
        infobox "$_PrepMount" "$_OnlyOne for EFI: $BOOT_PART\n" 1
    elif ! BOOT_PART="$(menubox "$_PrepMount" "$_SelUefiBody" 0 0 0 $PARTS)"; then
        return 1
    fi

    if grep -q 'fat' <<< "$(fsck -N "$BOOT_PART")"; then
        local msg="$_FormUefiBody $BOOT_PART $_FormUefiBody2"
        if yesno "$_PrepMount" "$msg" "Format $BOOT_PART" "Do Not Format" "no"; then
            format "$BOOT_PART" "vfat"
        fi
    else
        format "$BOOT_PART" "vfat"
    fi
    return 0
}

select_boot_partition()
{
    tput civis
    if ! BOOT_PART="$(menubox "$_PrepMount" "$_SelBiosBody" 0 0 0 "$_Skip" "-" $PARTS)" || [[ $BOOT_PART == "$_Skip" ]]; then
        BOOT_PART=""
    else
        if grep -q 'ext[34]' <<< "$(fsck -N "$BOOT_PART")"; then
            local msg="$_FormBiosBody $BOOT_PART $_FormUefiBody2"
            yesno "$_PrepMount" "$msg" "Format $BOOT_PART" "Skip Formatting" "no" && format "$BOOT_PART" "ext4"
        else
            format "$BOOT_PART" "ext4"
        fi
    fi
    return 0
}

select_root_partition()
{
    # if we used LUKS and no LVM or LUKS+LVM remove the relevant partition labels from the list
    if [[ $LUKS && ! $LVM ]]; then
        ROOT_PART="/dev/mapper/$LUKS_NAME"
        decr_count "$LUKS_PART"
    elif [[ $LVM ]]; then
        if [[ $LUKS ]]; then
            decr_count "$LUKS_PART"
        fi
        for part in $(printf "%s" "$GROUP_PARTS"); do
            decr_count "$part"
        done
        ROOT_PART=""
    fi

    if [[ $COUNT -eq 1 && ! $ROOT_PART ]]; then
        ROOT_PART="$(awk 'NF > 0 {print $1}' <<< "$PARTS")"
        infobox "$_PrepMount" "$_OnlyOne for root (/): $ROOT_PART\n" 1
    elif [[ ! $ROOT_PART || $LVM ]]; then
        tput civis
        if ! ROOT_PART="$(menubox "$_PrepMount" "$_SelRootBody" 0 0 0 $PARTS)"; then
            return 1
        fi
    else
        infobox "$_PrepMount" "\nUsing${LUKS} root partition: $ROOT_PART\n" 1
    fi

    select_filesystem "$ROOT_PART" || { ROOT_PART=""; return 1; }
    mount_partition "$ROOT_PART" || { ROOT_PART=""; return 1; }
    return 0
}

select_extra_partitions()
{
    while (( COUNT > 0 )); do
        tput civis
        local part
        if ! part="$(menubox "$_PrepMount " "$_ExtPartBody" 0 0 0 "$_Done" "-" $PARTS)" || [[ $part == "$_Done" ]]; then
            break
        elif ! select_filesystem "$part"; then
            break
            return 1
        elif ! select_mountpoint; then
            break
            return 1
        elif ! mount_partition "$part" "$EXTRA_MNT"; then
            break
            return 1
        fi
        EXTRA_MNTS="$EXTRA_MNTS $part: $EXTRA_MNT"
        [[ $EXTRA_MNT == "/usr" && $MKINIT_HOOKS != *usr* ]] && MKINIT_HOOKS="usr $MKINIT_HOOKS"
    done

    return 0
}
